unit Unit2;

interface

uses
  System.Variants, SysUtils;

type
  TCounterVarData = packed record
    VType: TVarType;
    Reserved1, Reserved2, Reserved3: Word;
    CounterValue: PInteger;
    Reserved4: LongWord;
  end;

  TVariantCounter = class(TInvokeableVariantType)
  private
    FCounter: Integer;
  public
    procedure Clear(var V: TVarData); override;

    function GetProperty(var Dest: TVarData; const V: TVarData;
      const Name: string): Boolean; override;
    function SetProperty(const V: TVarData; const Name: string;
      const Value: TVarData): Boolean; override;
    function DoFunction(var Dest: TVarData; const V: TVarData;
      const Name: string; const Arguments: TVarDataArray): Boolean; override;
    function DoProcedure(const V: TVarData; const Name: string;
      const Arguments: TVarDataArray): Boolean; override;
  end;

function NewVariantCounter: Variant;

implementation

var
  VariantCounter: TVariantCounter;

function NewVariantCounter: Variant;
begin
  VarClear(Result);
  TCounterVarData(Result).VType := VariantCounter.VarType;

  New(TCounterVarData(Result).CounterValue);
  TCounterVarData(Result).CounterValue^ := 0;
end;

{ TVariantCounter }

procedure TVariantCounter.Clear(var V: TVarData);
begin
  Dispose(TCounterVarData(V).CounterValue);
  inherited;
end;

function TVariantCounter.DoFunction(var Dest: TVarData; const V: TVarData;
  const Name: string; const Arguments: TVarDataArray): Boolean;
begin
  Result := False;

  if SameText(Name, 'Pow2') and (Length(Arguments) = 0) then
  begin
    Variant(Dest) := TCounterVarData(V).CounterValue^ * TCounterVarData(V).CounterValue^;
    Result := True;
  end;
end;

function TVariantCounter.DoProcedure(const V: TVarData; const Name: string;
  const Arguments: TVarDataArray): Boolean;
begin
  Result := False;

  if SameText(Name, 'Reset') and (Length(Arguments) = 0) then
  begin
    TCounterVarData(V).CounterValue^ := 0;
    Result := True;
  end;
end;

function TVariantCounter.GetProperty(var Dest: TVarData; const V: TVarData;
  const Name: string): Boolean;
begin
  if SameText(Name, 'Counter') then
  begin
    Variant(Dest) := TCounterVarData(V).CounterValue^;
    Result := True;
  end
  else
    Result := False;
end;

function TVariantCounter.SetProperty(const V: TVarData; const Name: string;
  const Value: TVarData): Boolean;
begin
  if SameText(Name, 'Counter') then
  begin
    TCounterVarData(V).CounterValue^ := Variant(Value);
    Result := True;
  end
  else
    Result := False;
end;

initialization

VariantCounter := TVariantCounter.Create;

finalization

VariantCounter.Free;

end.
